@@ -1,17 +1,17 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-
 from traceback import format_exception
 import readline
 import json
 import sys
 import os
 
+from rlpython.runtimes.command_runtime import CommandRuntime
+from rlpython.runtimes.python_runtime import PythonRuntime
+from rlpython.runtimes.shell_runtime import ShellRuntime
 from rlpython.utils.gc_utils import get_object_by_id
-from rlpython.command_runtime import CommandRuntime
-from rlpython.python_runtime import PythonRuntime
+from rlpython.templating import TemplatingEngine
 from rlpython.completion import Completer
 from rlpython.utils.strings import color
+from rlpython.aliases import Aliases
 from rlpython import VERSION_STRING
 
 DEFAULT_HISTORY_FILE = '~/.rlpython.history'
@@ -22,6 +22,7 @@ DEFAULT_PROMPT_PS2 = '... '
 
 DEFAULT_BANNER = """Python {} on {}
 rlpython {}
+Type '?' for help
 """.format(
     sys.version,
     sys.platform,
@@ -33,6 +34,30 @@ DEFAULT_VARIABLES = {
     'repeat_last_command_on_enter': False,
 }
 
+HELP_TEXT = """
+rlpython
+========
+
+rlpython is a GNU Readline based Python REPL, fully compatible with
+Python's syntax.
+
+
+Help / Inspection
+-----------------
+
+?             Show this help text
+object?       Inspect given object
+object??      Show help text of given object
+%edit object  Open the source code of given object if available
+help          Python's builtin help system
+
+
+REPL Commands
+-------------
+
+{commands}
+"""
+
 
 class Repl:
     class DOMAIN:
@@ -65,6 +90,9 @@ class Repl:
             **variables,
         }
 
+        # setup templating engine
+        self.templating_engine = TemplatingEngine(repl=self)
+
         # setup completion
         self.completer = Completer(repl=self)
 
@@ -89,6 +117,13 @@ class Repl:
         for command in commands:
             self.install_command(command)
 
+        self.shell_runtime = ShellRuntime(
+            repl=self,
+        )
+
+        # setup aliasses
+        self.aliases = Aliases(repl=self)
+
         # finish
         self.clear_line_buffer()
         self.setup()
@@ -185,10 +220,23 @@ class Repl:
         self.line_buffer = ''
 
     def validate_line_buffer(self):
-        if self.line_buffer.startswith('%'):
-            return self.command_runtime.validate_source(self.line_buffer)
+        try:
+            line_buffer = self.aliases.resolve(self.line_buffer)
 
-        return self.python_runtime.validate_source(self.line_buffer)
+            if line_buffer.startswith('%'):
+                return self.command_runtime.validate_source(line_buffer)
+
+            elif line_buffer.startswith('!'):
+                return self.shell_runtime.validate_source(line_buffer)
+
+            return self.python_runtime.validate_source(line_buffer)
+
+        except Exception as exception:
+            self.exit_code = 1
+
+            self.write_exception(exception, prefix='rlpython: ')
+
+            return False
 
     # I/O #####################################################################
     def write_warnings(self):
@@ -198,6 +246,13 @@ class Repl:
     def write_banner(self):
         self.write(self.banner)
 
+    def write_help(self):
+        text = HELP_TEXT.format(
+            commands=self.command_runtime.gen_help_text(),
+        )
+
+        self.write(text.strip() + '\n')
+
     def gen_prompt(self):
         prompt_color = 'green'
 
@@ -309,9 +364,28 @@ class Repl:
 
     def run(self, command):
         try:
-            if command.startswith('%'):
+            # aliasses
+            command = self.aliases.resolve(command)
+
+            # templating
+            if self.templating_engine.is_template(command):
+                command = self.templating_engine.render(command)
+                self.write(command)
+
+            # help
+            if command.strip() == '?':
+                self.write_help()
+                self.exit_code = 0
+
+            # repl commands
+            elif command.startswith('%'):
                 self.exit_code = self.command_runtime.run(command)
 
+            # shell commands
+            elif command.startswith('!'):
+                self.exit_code = self.shell_runtime.run(command)
+
+            # python code
             else:
                 self.exit_code = self.python_runtime.run(command)
 
